#pragma once

#include "../CodeWarCommon/CodeWarCommands.h"
#include "../CodeWarCommon/Thread.hpp"
using namespace Thread;
#include "RobotProperties.h"
#include <queue>

class Robot
{
public:
    Robot(void);
    ~Robot(void);

    bool initialize(unsigned short port);

    void actStupid(bool value = true) { m_actStupid = value; }

    // Status
    bool connected(void)
    {
        if( m_ConnectThread ) return( !m_ConnectThread->stopped() );
        else return( false );
    }

    FoodType getFoodStatus(void) { return( m_Properties.m_food ); }
    LifeType getLifeStatus(void) { return( m_Properties.m_life ); }
    bool attacked(void) { return( m_attacked ); }

    // Commands
    void rest(void);
    void moveForward(void);
    void moveBackward(void);
    void turnLeft(void);
    void turnRight(void);
    void eat(void);
    void fight(void);

    // "callback" functions
    void onTurn(void);                          // The robot has a turn now.
    void onHitWall(void);                       // The robot has hit a wall.
    void onRobotInSight(void);                  // The robot has another robot in sight.
    void onDead(void);                          // The robot just died.
    void onFoundFood(void);                     // The robot just found food.
    void onAttacked(void);                      // The robot is under fire.

private:    // class

    ////////////////////////////////////////////////////////////////////
    // An instance of this class will connect to the server.
    class ConnectThread : public CThread
    {
    public:
        ConnectThread(Robot * r, unsigned short port);

        void OnInitInstance();                  // Called before OnRunning()
        void OnRunning(void);                   // Start accepting client connections
        DWORD OnExitInstance();
        void Stop(void);
        bool stopped(void) { return( m_bStop ); }

        bool QueueCommand(char * cmd, int size);

    protected:
        bool m_bStop;                           // OnRunning() will continue
                                                //   until m_bStop == true
    private:
        void doTurn(CommandType & cmdType);
        bool SendCommand(void);

    private:
        Robot * m_Robot;
        unsigned short m_Port;
        SOCKET m_ClientSocket;
        bool m_hasTurn;

        HANDLE m_hMutex;
        typedef std::pair<char *, int> QueuedCommandPair;
        std::queue<QueuedCommandPair> m_QueuedCommands;

    };

private:    // data members
    unsigned short m_Port;
    ConnectThread * m_ConnectThread;

    RobotProperties m_Properties;

    NrType m_Nr;                        // Total nr of robots in the arena

    bool m_actStupid;                   // Use the AI if true

    // State variables
    bool m_wallHit;
    bool m_robotInSight;
    bool m_foodFound;
    bool m_attacked;
};
